@@ -26,6 +26,7 @@
 
 #include <osmocom/core/utils.h>
 #include <osmocom/core/socket.h>
+#include <osmocom/gsm/tlv.h>
 
 #include <sofia-sip/sip_status.h>
 #include <sofia-sip/su_log.h>
@@ -113,9 +114,21 @@ static void new_call(struct sip_agent *agent, nua_handle_t *nh,
 	struct sip_call_leg *leg;
 	const char *from = NULL, *to = NULL;
 	char ip_addr[INET6_ADDRSTRLEN];
+	bool xgcr_hdr_present = false;
+	uint8_t xgcr_hdr[28] = { 0 };
 
 	LOGP(DSIP, LOGL_INFO, "Incoming call(%s) handle(%p)\n", sip->sip_call_id->i_id, nh);
 
+	sip_unknown_t *unknown_header = sip->sip_unknown;
+	while (unknown_header != NULL) {
+		if (!strcmp("X-Global-Call-Ref", unknown_header->un_name)) {
+			osmo_hexparse(unknown_header->un_value, xgcr_hdr, sizeof(xgcr_hdr));
+			xgcr_hdr_present = true;
+			break;
+		}
+		unknown_header = unknown_header->un_next;
+	}
+
 	if (!sdp_screen_sdp(sip)) {
 		LOGP(DSIP, LOGL_ERROR, "No supported codec.\n");
 		nua_respond(nh, SIP_406_NOT_ACCEPTABLE, TAG_END());
@@ -131,6 +144,17 @@ static void new_call(struct sip_agent *agent, nua_handle_t *nh,
 		return;
 	}
 
+	/* Decode Decode the Global Call Reference (if present) */
+	if (xgcr_hdr_present) {
+		if (osmo_dec_gcr(&call->gcr, xgcr_hdr, sizeof(xgcr_hdr)) < 0) {
+			LOGP(DSIP, LOGL_ERROR, "Failed to parse X-Global-Call-Ref.\n");
+			nua_respond(nh, SIP_406_NOT_ACCEPTABLE, TAG_END());
+			nua_handle_destroy(nh);
+			return;
+		}
+		call->gcr_present = true;
+	}
+
 	if (sip->sip_to)
 		to = sip->sip_to->a_url->url_user;
 	if (sip->sip_from)
@@ -214,8 +238,8 @@ static void sip_handle_reinvite(struct sip_call_leg *leg, nua_handle_t *nh, cons
 	}
 
 	LOGP(DSIP, LOGL_DEBUG, "pre re-INVITE have IP:port (%s:%u)\n",
-	     osmo_sockaddr_ntop((struct sockaddr*)&leg->base.addr, ip_addr),
-	     osmo_sockaddr_port((struct sockaddr*)&leg->base.addr));
+	     osmo_sockaddr_ntop((struct sockaddr*)&prev_addr, ip_addr),
+	     osmo_sockaddr_port((struct sockaddr*)&prev_addr));
 
 	if (mode == sdp_sendonly) {
 		/* SIP side places call on HOLD */
@@ -234,10 +258,16 @@ static void sip_handle_reinvite(struct sip_call_leg *leg, nua_handle_t *nh, cons
 		     osmo_sockaddr_ntop((struct sockaddr*)&leg->base.addr, ip_addr),
 		     osmo_sockaddr_port((struct sockaddr*)&leg->base.addr));
 		if (osmo_sockaddr_cmp((struct osmo_sockaddr *)&prev_addr,
-				      (struct osmo_sockaddr *)&leg->base)) {
-			LOGP(DSIP, LOGL_INFO, "re-INVITE changes media connection.\n");
+				      (struct osmo_sockaddr *)&leg->base.addr)) {
+			LOGP(DSIP, LOGL_INFO, "re-INVITE changes media connection to %s:%u\n",
+			     osmo_sockaddr_ntop((struct sockaddr*)&leg->base.addr, ip_addr),
+			     osmo_sockaddr_port((struct sockaddr*)&leg->base.addr));
 			if (other->update_rtp)
 				other->update_rtp(leg->base.call->remote);
+		} else {
+			LOGP(DSIP, LOGL_INFO, "re-INVITE does not change media connection (%s:%u)\n",
+			     osmo_sockaddr_ntop((struct sockaddr*)&prev_addr, ip_addr),
+			     osmo_sockaddr_port((struct sockaddr*)&prev_addr));
 		}
 		sdp = sdp_create_file(leg, other, sdp_sendrecv);
 	}
@@ -599,6 +629,19 @@ static int send_invite(struct sip_agent *agent, struct sip_call_leg *leg,
 				agent->app->sip.remote_port);
 	char *sdp = sdp_create_file(leg, other, sdp_sendrecv);
 
+	/* Encode the Global Call Reference (if present) */
+	char *x_gcr = NULL;
+
+	if (leg->base.call->gcr_present) {
+		struct msgb *msg = msgb_alloc(16, "SIP GCR");
+
+		if (msg != NULL && osmo_enc_gcr(msg, &leg->base.call->gcr) > 0)
+			x_gcr = talloc_asprintf(leg, "X-Global-Call-Ref: %s", msgb_hexdump(msg));
+		else
+			LOGP(DSIP, LOGL_ERROR, "Failed to encode GCR for leg(%p)\n", leg);
+		msgb_free(msg);
+	}
+
 	leg->state = SIP_CC_INITIAL;
 	leg->dir = SIP_DIR_MT;
 	nua_invite(leg->nua_handle,
@@ -606,6 +649,7 @@ static int send_invite(struct sip_agent *agent, struct sip_call_leg *leg,
 			SIPTAG_TO_STR(to),
 			NUTAG_MEDIA_ENABLE(0),
 			SIPTAG_CONTENT_TYPE_STR("application/sdp"),
+			TAG_IF(x_gcr, SIPTAG_HEADER_STR(x_gcr)),
 			SIPTAG_PAYLOAD_STR(sdp),
 			TAG_END());
 
@@ -613,6 +657,7 @@ static int send_invite(struct sip_agent *agent, struct sip_call_leg *leg,
 	talloc_free(from);
 	talloc_free(to);
 	talloc_free(sdp);
+	talloc_free(x_gcr);
 	return 0;
 }
 
